@@ -178,8 +178,10 @@ class Reader(AbstractReader):
                     continue
 
                 for wire_type in element['properties']['wires']:
+                    found_wire = False
                     for db_wire in self.equipment_data['wires']:
                         if db_wire['nameclass'] == wire_type:
+                            found_wire = True
                             wire = Wire(model)
                             wire.nameclass = wire_type.replace(' ','_').replace('/','-')
                             if 'OH' in wire_type:
@@ -199,6 +201,8 @@ class Reader(AbstractReader):
                             wire.X = float(db_wire['x'])*0.3048
                             wire.Y = float(db_wire['height'])*0.3048
                             all_wires.append(wire)
+                    if not found_wire:
+                        raise ValueError(f'No wire found in catalog for {wire_type}')
                 line.wires = all_wires
 
 
@@ -242,13 +246,13 @@ class Reader(AbstractReader):
                 node = Node(model)
                 node.name = element['properties']['id']
                 if node.name in self.substations:
-                    node.nominal_voltage = 13200
+                    node.nominal_voltage = 13200 #placeholder voltage
                     node.is_substation_connection = True
                     node.setpoint = 1.0
                     node.name = 'source'
                     meta = Feeder_metadata(model)
                     meta.headnode = 'source'
-                    meta.nominal_voltage = 13200
+                    meta.nominal_voltage = 13200 #placeholder voltage
                     meta.name = 'urbanopt-feeder'
                     powersource = PowerSource(model)
                     powersource.is_sourcebus = True
@@ -283,6 +287,7 @@ class Reader(AbstractReader):
                 else:
                     transformer_panel_map[element['properties']['DSId']] = [element['properties']['id']]
 
+        source_voltages = set() 
         for element in self.geojson_content["features"]:
             if 'properties' in element and 'district_system_type' in element['properties'] and element['properties']['district_system_type'] == 'Transformer':
                 transformer_id = element['properties']['id']
@@ -293,8 +298,10 @@ class Reader(AbstractReader):
                     if len(transformer_panel_map[transformer_id]) >2:
                         print("Warning - the transformer "+transformer_id+" should have a from and to element - "+str(len(transformer_panel_map[transformer_id]))+" junctions on the transformer")
                     if len(transformer_panel_map[transformer_id]) >=2:
+                        found_transfomer = False
                         for db_transformer in self.equipment_data['transformer_properties']:
                             if element['properties']['equipment'][0] == db_transformer['nameclass']:
+                                found_transfomer = True
                                 transformer.from_element = transformer_panel_map[transformer_id][0]
                                 transformer.to_element = transformer_panel_map[transformer_id][1] #NOTE: Need to figure out correct from and to directions here.
                                 transformer.name = transformer_id
@@ -323,6 +330,7 @@ class Reader(AbstractReader):
                                     windings[i].rated_power = float(db_transformer['kva'])*1000
                                     if i<1:
                                         windings[i].nominal_voltage = float(db_transformer['high_voltage'])*1000
+                                        source_voltages.add(windings[i].nominal_voltage)
                                         if transformer.is_center_tap:
                                             windings[i].nominal_voltage = windings[i].nominal_voltage/(3**0.5)
                                         windings[i].connection_type = connection_map[connections[0]]
@@ -334,8 +342,20 @@ class Reader(AbstractReader):
                                         windings[i].voltage_type = 1
                                         windings[i].resistance = float(db_transformer['resistance'])
                                 transformer.windings = windings
+                        if not found_transfomer:
+                            raise ValueError(f'No transfomer found in catalog for {element["properties"]["equipment"][0]}')
 
 
+        # Note that the source voltage is set to be the highest side of a transformer that is used
+        if len(source_voltages) == 1:
+            source_voltage = source_voltages.pop()
+            model.set_names()
+            model['source'].nominal_voltage = source_voltage
+            model['ps_source'].nominal_voltage = source_voltage
+            model['urbanopt-feeder'].nominal_voltage = source_voltage
+
+        else:
+            raise ValueError('Problem setting source voltage. No high transformer values or non-unique high side voltages. Using defaul of 13.2kV')
         return 1
 
     def parse_capacitors(self, model, **kwargs):
@@ -358,70 +378,12 @@ class Reader(AbstractReader):
         model.set_names()
         network = Network()
         network.build(model,source="source")
-        number_components = nx.number_connected_components(network.graph)
-        self.deleted_elements = None
-        if not number_components==1:
-            updated_model=model
-            source_component = None
-            c_id = 0
-            print(f"Warning - network is disconnected with {number_components} components")
-            components = []
-            for component in nx.connected_components(network.graph):
-                nodes = {}
-                for node in component:
-                    if node in model.model_names and isinstance(model[node],Node):
-                        nodes[node] = (model[node].positions[0].lat,model[node].positions[0].long)
-                    if node in model.model_names and isinstance(model[node],PowerSource):
-                        source_component = c_id
-                components.append(nodes)
-                c_id+=1
-            if source_component is None:
-                raise ValueError("No substation in geojson_file")
-            seen = set([source_component])
-            self.deleted_elements = {}
-            modifier = Modifier()
-            while len(seen) < len(components): # Incrementally add closest component to the component connected to the substation
-                closest_dist = 10000000000000000
-                closest_component = None
-                closest_node_pair = None
-                connecting_component = None
-                for c_id in range(len(components)):
-                    if c_id in seen:
-                        continue
-                    for c_id2 in seen:
-                        for node1_name,node1 in components[c_id].items():
-                            for node2_name,node2 in components[c_id2].items():
-                                if not node2_name in self.deleted_elements: #So don't attach to a node that's already been deleted
-                                    distance = math.sqrt((node1[0]-node2[0])**2 + (node1[1]-node2[1])**2)
-                                    if distance < closest_dist:
-                                        closest_dist = distance
-                                        closest_node_pair = (node1_name,node2_name)
-                                        closest_component = c_id
-                                        connecting_component = c_id2
-
-                node_to_remove = closest_node_pair[0]
-                node_to_use = closest_node_pair[1]
-                for i in model.models:
-                    if hasattr(i,'from_element') and i.from_element == node_to_remove:
-                        i.from_element = node_to_use
-                    if hasattr(i,'to_element') and i.to_element == node_to_remove:
-                        i.to_element = node_to_use
-                    if hasattr(i,'connecting_element') and i.connecting_element == node_to_remove:
-                        i.connecting_element = node_to_use
-                model = modifier.delete_element(model,model[node_to_remove])
-                print(f'Deleting node {node_to_remove}',flush=True)
-                seen.add(closest_component)
-                self.deleted_elements[node_to_remove] = node_to_use
-
-            model.set_names()
-            network = Network()
-            network.build(model,source="source")
-
 
         building_map = {}
         for element in self.geojson_content["features"]:
             if 'properties' in element and 'type' in element['properties'] and 'buildingId' in element['properties'] and element['properties']['type'] == 'ElectricalJunction':
                 building_map[element['properties']['buildingId']] = element['properties']['id']
+        disconnected_loads = []
         for element in self.geojson_content["features"]:
             if 'properties' in element and 'type' in element['properties'] and element['properties']['type'] == 'Building':
                 id_value = element['properties']['id']
@@ -431,11 +393,12 @@ class Reader(AbstractReader):
                 connecting_element = building_map[id_value]
                 load = Load(model)
                 load.name = id_value
-                if self.deleted_elements is not None:
-                    while connecting_element in self.deleted_elements:
-                        connecting_element = self.deleted_elements[connecting_element]
                 load.connecting_element = connecting_element
-                upstream_transformer_name = network.get_upstream_transformer(model,connecting_element)
+                upstream_transformer_name = None
+                try:
+                    upstream_transformer_name = network.get_upstream_transformer(model,connecting_element)
+                except:
+                    disconnected_loads.append(element['properties']['id']) #Caused by elements not being connected
                 if upstream_transformer_name is not None:
                     upstream_transformer = model[upstream_transformer_name]
                     is_center_tap = upstream_transformer.is_center_tap
@@ -444,6 +407,7 @@ class Reader(AbstractReader):
                     print(f'Warning - Load {load.name} is incorrectly connected',flush=True)
 
                 load_path = os.path.join(self.load_folder,id_value,'feature_reports')
+                load_multiplier = 1000
                 if os.path.exists(load_path): #We've found the load data
 
                     load_data = None
@@ -454,6 +418,11 @@ class Reader(AbstractReader):
                     else:
                         load_data = pd.read_csv(os.path.join(load_path,'default_feature_report.csv'),header=0)
                         load_column = 'Net Power(kW)'
+                        if not load_column in load_data:
+                            load_column = 'Net Power(W)'
+                            load_multiplier = 1
+                        if not load_column in load_data:
+                            raise ValueError("Neither of the columns 'Net Power(W)' or 'Net Power(kW)' found in default_feature_report.csv'")
                     max_load = max(load_data[load_column])
 
                     phases = []
@@ -468,8 +437,8 @@ class Reader(AbstractReader):
                         phase_load = PhaseLoad(model)
                         phase_load.phase = phase
                         power_factor = 0.95
-                        phase_load.p = max_load/len(phases)
-                        phase_load.q = phase_load.p * ((1/power_factor-1)**0.5)
+                        phase_load.p = max_load/len(phases) * load_multiplier
+                        phase_load.q = phase_load.p * ((1/power_factor-1)**0.5) 
                         phase_loads.append(phase_load)
                     load.phase_loads = phase_loads
                     if self.is_timeseries:
@@ -501,6 +470,10 @@ class Reader(AbstractReader):
                     print('Load information missing for '+id_value,flush=True)
 
 
+        if len(disconnected_loads) > 1:
+            all_disconnected = ','.join(disconnected_loads)
+            #raise ValueError('The following loads have connection problems: '+all_disconnected)
+            print('The following loads have connection problems: '+all_disconnected)
         return 1
 
 
@@ -527,9 +500,6 @@ class Reader(AbstractReader):
                     print(f'Warning - {id_value} missing from building object. Skipping...',flush=True)
                     continue
                 connecting_element = building_map[id_value]
-                if self.deleted_elements is not None:
-                    while connecting_element in self.deleted_elements:
-                        connecting_element = self.deleted_elements[connecting_element]
                 try:
                     feature_data = self.get_feature_data(os.path.join(self.load_folder,id_value,'feature_reports','feature_report_reopt.json'))
                 except Exception as e:
